%CARTEETSTD_SST permet de visualiser les cartes mensuelles moyennes
% de temperature de surface       
% ainsi que les cartes annuelles moyennes et les cart type
% correspondants

% Date: Avril-Mai 2008
% Auteur: juliette.mignot@locean-ipsl.upmc.fr
% Contexte Ecole d'ete ENVI-STAT 2008
%+
%
% module
% ======
%
% ``carteetstd_SST``
%
% DESCRIPTION
% ===========
%
% ``carteetstd_SST`` permet de visualiser les cartes mensuelles moyennes
% de temperature de surface
% ainsi que les cartes annuelles moyennes et les cart type
% correspondants.
%
% ``carteetstd_SST`` lit les fichiers 
% ``../DONNEES/lsmask_tropatl_PacandMedblancs.mat``
% et ``../DONNEES/skt.mon.tropatl.mat``.
%
% ``carteetstd_SST`` calcule les moyennes mensuelles d'une part et
% les moyennes interannuelles d'autre part de la SST dans
% l'Atlantique tropical  partir d'un champ mensuel de 60 ans
% fourni en entre.
%
% ``carteetstd_SST`` affiche  l'cran les 12 valeurs de moyennes
% mensuelles (figure 1), une slection de 12 valeurs annuelles
% moyennes (figure 2) et les cart-type mensuels et interannuels
% (figure 3).
%
% ``carteetstd_SST`` sauve ces images dans ``./carteetstd_SST_[123].ps``.
%
% EXAMPLES
% ========
%
% ::
%
% >> addpath('../PROGRAMMES/')
% >> tpacpandreg_startup
% >> carteetstd_SST
%
% SEE ALSO
% ========
%
% tpacpandreg_startup.m_
%
% .. _tpacpandreg_startup.m : tpacpandreg_startup.m.html
%
% initfig.m_
%
% .. _initfig.m : initfig.m.html
%
% whorldmap.m_
%
% .. _whorldmap.m : whorldmap.m.html
%
% colorbartype.m_
%
% .. _colorbartype.m : colorbartype.m.html
%
% contlab.m_
%
% .. _contlab.m : contlab.m.html
%
%
% KNOWN PROBLEMS
% ==============
%
% pb avec les diacritiques  l'impression de .ps dans le titres des figures 
% sous octave :  se transforme en o barr.
%
% - fplod 2009-09-21T12:59:35Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   *  octave 3.2.2 line 0: Internal error : unknown tic type
%
% TODO
% ====
%
% improve description
%
% is there any way to call worldmap only once or at least read coast.m only
% once
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% - fplod 2009-09-21T12:12:06Z aedon.locean-ipsl.upmc.fr (Darwin)
% 
%   * octave 3.2.2 contourf problem is not solved (see KNOWN PROBLEMS) but a
%     workaround allows (``contour`` instead of ``contourf``) to run this
%     script till end.
%
% - fplod 2009-08-26T09:56:51Z aedon.locean-ipsl.upmc.fr (Darwin)
%   
%   * nom des mois  la place du numro dans les titres
%   * remplacement de ``eps`` par ``ps`` pour pouvoir imprimer les figures 
%     produites par ``octave``
%
%
% - jmignot 2009-08-25
%
%   * suppression d'appel  contlab et colorbartype sous octave
%
% - fplod 2009-08-25T10:40:16Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * modif commentaires lie au dernier chgt de Juliette
%
% - jmignot 2009-08-25
%
%   * remplacement de ``colorbar`` par ``colorbartype``
%     parce pas beau ni en matlab ni en octave (dgrad, noir et blanc dans 
%     les .jpg) 
%   * contraintes sur les limites x et y de axes car matlab et octave ne
%     fixent pas les mme limites par dfaut
%   * utisation de ``contlab`` pour ++
% 
% - fplod 2009-08-17T12:39:51Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add header
%   * rpertoire de donnes relatifs au rpertoire courant
%   * suppression de ``clear all`` prsent dans ``tpacpandreg_startup.m``
%   * contournement pb octave pour ``contourf`` :
%     Dans le fil de discussion [Pkg-octave-devel] Bug#492223: octave3.0: contourf only accepts vector X and Y if same+++,
%     je comprends que j'ai affaire  un bug de octave 3.0 (donc  surveiller
%     pour les prochaines versions) qui ne veut pas faire de contourf si les
%     variables X et Y n'ont pas la mme dimension et c'est le cas ici
%     (lat = 42x1, lon= 70x1 vue avec size).
%     http://lists.alioth.debian.org/pipermail/pkg-octave-devel/2008-July/004672.html
%   * sauvegarde de la figure dans ``./carteetstd_SST.eps``
%   * ajout de ``clf`` aprs les appels  ``figure`` sinon sous 
%     octave (pas sous matlab?) on retrouve des subplots de la figure 2 dans
%     la figure 3 !
%
%-

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%              CHARGEMENT ET PREPARATION DES DONNEES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nannees=60;

repertoire=[ '..' filesep 'DONNEES' filesep ] % a adapter a chaque cas: chemin d acces aux donnees

%Mask
%------
file='lsmask_tropatl_PacandMedblancs.mat'
   load([repertoire file]);
% Data
%--------
  repertoire2=[repertoire 'MAT' filesep] % a adapter a chaque cas: chemin d acces aux donnees
  file=['skt.mon.tropatl.mat']
  load([repertoire2 file]); % chargement des donnees
  tab=permute(tab,[2 3 1]); % permutation des dimensions pour organiser le tableau sous la forme [lat lon temps]

% Application du masque
%----------------------
% le masque donne les positions des continents et des oceans
mask3D=repmat(mask,[1 1 size(tab,3)]);
tab(mask3D~=0)=NaN;

%---------------------------------------------------
%  MANIPULATIONS TEMPORELLES: MOYENNES 
%---------------------------------------------------
tab=reshape(tab,size(tab,1),size(tab,2),12,nannees); %organisation du tableau sous la forme [lat lon mois annes]

% Construction de moyennes annuelles
%---------------------------------------
tab_an=squeeze(mean(tab,3));

% Construction de moyennes mensuelles
%---------------------------------------
tab_mens=squeeze(mean(tab,4));

%calcul des cart-types
%------------------------
%cart-type mensuel moyen: 
std_mens=squeeze(std(tab_mens,0,3));

%cart-type interannuel moyen: 
std_an=squeeze(std(tab_an,0,3));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%               TRACE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%parametres geographiques du graphe
  geo=[-100 20 -30 30]; %[lonmin lonmax latmin latmax];
  yti=[-30:10:30]; xti=[-100:30:20]; %labels de latitude et de longitude

		 
% Variabilit saisonnire
%-------------------------
%niveaux de couleurs
lev=[0:1:30];%contours a tracer, en unite du champ a dessiner, ici
             %la SST en degC
clip_lev=[20 30];%definit les limites des couleurs: bleu fonce au
                 %rouge fonce


ifigure=1;
figure(ifigure); 
clf();
initfig; 
for iplot=1:12
  subplot(4,3,iplot)
  if (run_octave == 0)
   x=lon;
   y=lat;
  else
   [x,y]=meshgrid(lon,lat);
  end
  z=squeeze(tab_mens(:,:,iplot));
  if (run_octave == 0)
   contourf(x,y,z,lev)
  else
   switch OCTAVE_VERSION
    case {'3.0.2','3.0.5'}
     contourf(x,y,z,lev)
    case {'3.2.2'}
     contour(x,y,z,lev)
    otherwise
     contourf(x,y,z,lev)
   end
  end
  caxis(clip_lev)
  hold on
  whorldmap

  % limites geographies et labels de longitude et de latitude
  ylim([geo(3) geo(4)])
  xlim([geo(1) geo(2)])
  set(gca, 'Xtick', xti, 'Ytick', yti)
  if (run_octave == 0)
   [longstr, latstr] = contlab(xti, yti);
   set(gca, 'Xticklabel', longstr, 'Yticklabel', latstr)
  end;
  if iplot>=10
    xlabel('longitude')
  end;
  if iplot==1|iplot==4|iplot==7|iplot==10
    ylabel('latitude')
  end;
  
  %titre: nom de chaque mois
  cmois = datestr(datenum([2000 iplot 1 00 00 00]), 'mmm');
  title(cmois);
  clear cmois;
  
end;

%barre de couleur
if (run_octave == 0)
  pos=[.2 .03 .6 .013];
  hpal=jet(100); %carte des couleurs: du bleu au rouge par defaut.
  colorbartype(pos,lev,1,clip_lev,hpal,0);
  title('SST mensuelle moyenne en degC')
else
  colorbar('East') %colorbar par defaut
end;

% sauvegarde de la figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./',mfilename,'_',num2str(ifigure),'.',printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;


% Variabilit interannuelle
%-------------------------
%niveaux de couleurs
lev=[0:1:30];%contours a tracer, en unite du champ a dessiner, ici
             %la SST en degC
clip_lev=[20 30];%definit les limites des couleurs: bleu fonce au
                 %rouge fonce

% Cartes individuelles

ifigure=ifigure+1;
figure(ifigure);
clf();
initfig; 
for iplot=1:5:60  
  subplot(4,3,((iplot-1)/5)+1)  
    if (run_octave == 0)
   x=lon;
   y=lat;
  else
   [x,y]=meshgrid(lon,lat);
  end
  z=squeeze(tab_an(:,:,iplot));
  if (run_octave == 0)
   contourf(x,y,z,lev)
  else
   switch OCTAVE_VERSION
    case {'3.0.2','3.0.5'}
     contourf(x,y,z,lev)
    case {'3.2.2'}
     contour(x,y,z,lev)
    otherwise
     contourf(x,y,z,lev)
   end
  end
  caxis(clip_lev)
  hold on
  whorldmap

  % limites geographies et labels de longitude et de latitude
  ylim([geo(3) geo(4)])
  xlim([geo(1) geo(2)])
  set(gca, 'Xtick', xti, 'Ytick', yti)
  if (run_octave == 0)
   [longstr, latstr] = contlab(xti, yti);
   set(gca, 'Xticklabel', longstr, 'Yticklabel', latstr)
  end;
  if iplot>=10
    xlabel('longitude')
  end;
  if iplot==1|iplot==4|iplot==7|iplot==10
    ylabel('latitude')
  end;

  %titre: numro de l'anne
  title(['annee ' num2str(iplot+1948-1)])
end; 

%barre de couleur
if (run_octave == 0)
  pos=[.2 .03 .6 .013];
  hpal=jet(100); %carte des couleurs: du bleu au rouge par defaut.
  colorbartype(pos,lev,1,clip_lev,hpal,0);
  title('SST mensuelle moyenne en degC')
else
  colorbar('East') %colorbar par defaut
end;

% sauvegarde de la figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./',mfilename,'_',num2str(ifigure),'.',printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;


% cart type
%-----------


%trace
ifigure=ifigure+1;
figure(ifigure);
clf();
initfig; 

%cart-type mensuel moyen

%niveaux de couleurs
lev=[0:.4:5];
clip_lev=[0 3.6];

subplot(2,2,1)
if (run_octave == 0)
 x=lon;
 y=lat;
else
 [x,y]=meshgrid(lon,lat);
end
z=std_mens;
if (run_octave == 0)
 contourf(x,y,z,lev)
else
 switch OCTAVE_VERSION
  case {'3.0.2','3.0.5'}
   contourf(x,y,z,lev)
  case {'3.2.2'}
   contour(x,y,z,lev)
  otherwise
   contourf(x,y,z,lev)
 end
end
caxis([clip_lev])
hold on
whorldmap
% limites geographies et labels de longitude et de latitude
ylim([geo(3) geo(4)])
xlim([geo(1) geo(2)])
set(gca, 'Xtick', xti, 'Ytick', yti)
if (run_octave == 0)
 [longstr, latstr] = contlab(xti, yti);
 set(gca, 'Xticklabel', longstr, 'Yticklabel', latstr)
end;
xlabel('longitude')
ylabel('latitude')

title('ecart-type mensuel moyen')
%barre de couleur
if (run_octave == 0)
  pos=[.1 .5 .35 .013];
  hpal=jet(100); %carte des couleurs: du bleu au rouge par defaut.
  colorbartype(pos,lev,1,clip_lev,hpal,0);
  title('degC')
else
  colorbar('East') %colorbar par defaut
end;



%cart-type interannuel  moyen

%niveaux de couleurs
lev=[0:.05:.7];%la variabilite interannuelle est moins forte que la
               %variabilite saisonniere moyenne. 
clip_lev=[0.2 .55];

subplot(2,2,2)
z=std_an;
if (run_octave == 0)
 contourf(x,y,z,lev)
else
 switch OCTAVE_VERSION
  case {'3.0.2','3.0.5'}
   contourf(x,y,z,lev)
  case {'3.2.2'}
   contour(x,y,z,lev)
  otherwise
   contourf(x,y,z,lev)
 end
end
caxis(clip_lev)
hold on
whorldmap
% limites geographies et labels de longitude et de latitude
ylim([geo(3) geo(4)])
xlim([geo(1) geo(2)])
set(gca, 'Xtick', xti, 'Ytick', yti)
if (run_octave == 0)
 [longstr, latstr] = contlab(xti, yti);
 set(gca, 'Xticklabel', longstr, 'Yticklabel', latstr)
end;
xlabel('longitude')
ylabel('latitude')

title('cart-type interannuel')
%barre de couleur
if (run_octave == 0)
  pos=[.55 .5 .35 .013];
  hpal=jet(100); %carte des couleurs: du bleu au rouge par defaut.
  colorbartype(pos,lev,1,clip_lev,hpal,0);
  title('degC')
else
  colorbar('East') %colorbar par defaut
end;

% sauvegarde de la figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./',mfilename,'_',num2str(ifigure),'.',printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;
